package apps;

import data.*;
import imaging.*;
import misc.*;
import tools.*;

import java.io.IOException;

/**
 * Converts Camino dts to image format.
 *
 * @author Philip Cook
 * @version $Id$
 */
public class EigToImage extends Executable {
    

    private ImageHeader header;

    private int xDataDim;
    private int yDataDim;
    private int zDataDim;

    private int numComps;

    private DataSource data;

    private static String outputRoot = "eig_";
    
    /** Header for diffusion data, used to initialize tensor header.*/
    private static String headerFile = null;

    public EigToImage(String[] args) {
        super(args);
    }
    
    public void initDefaultVals() {	
        xDataDim = 0;
        yDataDim = 0;
        zDataDim = 0;

        header = null;

        data = null;

        numComps = 1;
    }
    
    public void initOptions(String[] args) {
        
        CL_Initializer.inputDataType = "double";
        OutputManager.outputDataType = "float";
        
        // Default 1 component, can be overriden by user
        CL_Initializer.maxTensorComponents = 1;

        CL_Initializer.CL_init(args);
        
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-outputroot")) {
                outputRoot = args[i+1];
                CL_Initializer.markAsParsed(i, 2);
            }
        }
        
        CL_Initializer.checkParsing(args);
        
        numComps = CL_Initializer.maxTensorComponents;

        CL_Initializer.initInputSpaceAndHeaderOptions();

        header = CL_Initializer.headerTemplate;
        
        xDataDim = header.xDataDim();
        yDataDim = header.yDataDim();
        zDataDim = header.zDataDim();

        data = ExternalDataSource.getDataSource(CL_Initializer.inputFile, 12 * CL_Initializer.maxTensorComponents,
                                                CL_Initializer.inputDataType);

    }
    
    public void initVariables() {
    }
	
    public void execute(OutputManager om) {	
      
           
        // data array for image, in Nifti format
        double[][][][][] e1 = new double[numComps][xDataDim][yDataDim][zDataDim][3];
        double[][][][][] e2 = new double[numComps][xDataDim][yDataDim][zDataDim][3];
        double[][][][][] e3 = new double[numComps][xDataDim][yDataDim][zDataDim][3];
            
        double[][][][] l1 = new double[numComps][xDataDim][yDataDim][zDataDim];
        double[][][][] l2 = new double[numComps][xDataDim][yDataDim][zDataDim];
        double[][][][] l3 = new double[numComps][xDataDim][yDataDim][zDataDim];

        double[][][][] md = new double[numComps][xDataDim][yDataDim][zDataDim];
        double[][][][] fa = new double[numComps][xDataDim][yDataDim][zDataDim];
        
        for (int k = 0; k < zDataDim; k++) {
            for (int j = 0; j < yDataDim; j++) {
                for (int i = 0; i < xDataDim; i++) {
                    double[] voxel = data.nextVoxel();
                    
                    for (int c = 0; c < numComps; c++) {
                        double compL1 = voxel[c*12];
                        e1[c][i][j][k][0] = voxel[c*12+1];
                        e1[c][i][j][k][1] = voxel[c*12+2];
                        e1[c][i][j][k][2] = voxel[c*12+3];
                        
                        double compL2 = voxel[c*12+4];
                        e2[c][i][j][k][0] = voxel[c*12+5];
                        e2[c][i][j][k][1] = voxel[c*12+6];
                        e2[c][i][j][k][2] = voxel[c*12+7];
                        
                        double compL3 = voxel[c*12+8];
                        e3[c][i][j][k][0] = voxel[c*12+9];
                        e3[c][i][j][k][1] = voxel[c*12+10];
                        e3[c][i][j][k][2] = voxel[c*12+11];

                        double compMD = (compL1 + compL2 + compL3) / 3.0;

                        l1[c][i][j][k] = compL1;
                        l2[c][i][j][k] = compL2;
                        l3[c][i][j][k] = compL3;

                        md[c][i][j][k] = compMD;
                        
                        fa[c][i][j][k] = Math.sqrt(1.5 * ((compL1 - compMD) * (compL1 - compMD) + 
                                                          (compL2 - compMD) * (compL2 - compMD) + 
                                                          (compL3 - compMD) * (compL3 - compMD)) / 
                                                   (compL1 * compL1 + compL2 * compL2 + compL3 * compL3));
                        

                    }
                }
            }
            
        }
                
        if (numComps > 1) {
            for (int c = 0; c < numComps; c++) {
                header.writeVectorImage(e1[c], outputRoot + "V1_" + (c+1));
                header.writeVectorImage(e2[c], outputRoot + "V2_" + (c+1));
                header.writeVectorImage(e3[c], outputRoot + "V3_" + (c+1));

                header.writeScalarImage(l1[c], outputRoot + "L1_" + (c+1));
                header.writeScalarImage(l2[c], outputRoot + "L2_" + (c+1));
                header.writeScalarImage(l3[c], outputRoot + "L3_" + (c+1));
                
                header.writeScalarImage(md[c], outputRoot + "MD_" + (c+1));
                header.writeScalarImage(fa[c], outputRoot + "FA_" + (c+1));
            }
        }
        else {
            header.writeVectorImage(e1[0], outputRoot + "V1");
            header.writeVectorImage(e2[0], outputRoot + "V2");
            header.writeVectorImage(e3[0], outputRoot + "V3");

            header.writeScalarImage(l1[0], outputRoot + "L1");
            header.writeScalarImage(l2[0], outputRoot + "L2");
            header.writeScalarImage(l3[0], outputRoot + "L3");
            
            header.writeScalarImage(md[0], outputRoot + "MD");
            header.writeScalarImage(fa[0], outputRoot + "FA");
        }
                                    
    }  
 
}
